﻿using Hotline.KnowledgeBase;
using Hotline.Permissions;
using Hotline.Share.Dtos.Knowledge;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class KnowledgeCommonController : BaseController
    {
        private readonly IKnowledgeTypeRepository _knowledgeTypeRepository;
        private readonly IKnowledgeTypeDomainService _knowledgeTypeDomainService;
        private readonly IKnowledgeStandardRepository _knowledgeStandardRepository;
        private readonly IMapper _mapper;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeTypeRepository"></param>
        /// <param name="knowledgeTypeDomainService"></param>
        /// <param name="knowledgeStandardRepository"></param>
        /// <param name="mapper"></param>
        public KnowledgeCommonController(IKnowledgeTypeRepository knowledgeTypeRepository, IKnowledgeTypeDomainService knowledgeTypeDomainService,
            IKnowledgeStandardRepository knowledgeStandardRepository,
            IMapper mapper)
        {
            _knowledgeTypeRepository = knowledgeTypeRepository;
            _knowledgeTypeDomainService = knowledgeTypeDomainService;
            _knowledgeStandardRepository = knowledgeStandardRepository;
            _mapper = mapper;
        }

        #region 知识分类

        /// <summary>
        ///知识分类- 新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.AddKnowledgeType)]
        [HttpPost("add")]
        public async Task<string> AddType([FromBody] AddKnowledgeTypeDto dto)
        {
            return await _knowledgeTypeDomainService.AddType(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        ///知识分类- 编辑
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.UpdateKnowledgeType)]
        [HttpPut("update")]
        public async Task UpdateType([FromBody] UpdateKnowledgeTypeDto dto)
        {
            await _knowledgeTypeDomainService.UpdateType(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识分类-新增、编辑初始化
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="ParentId"></param>
        /// <returns></returns>
        [HttpGet("init")]
        public async Task<KnowledgeTypeDto> InitType(string Id, string ParentId)
        {
            return await _knowledgeTypeDomainService.InitType(Id, ParentId, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识分类-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("info/{Id}")]
        public async Task<KnowledgeType> GetType(string Id)
        {
            var types = await _knowledgeTypeRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (types is null)
                throw UserFriendlyException.SameMessage("查询失败！");
            return types;
        }

        /// <summary>
        /// 知识分类-删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.RemoveKnowledgeType)]
        [HttpDelete("remove")]
        public async Task RemoveType(string Id)
        {
            await _knowledgeTypeDomainService.RemoveType(Id, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识分类-启用、禁用切换
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeTypeChangeState)]
        [HttpPut("changestate")]
        public async Task ChangeStateType(string Id)
        {
            await _knowledgeTypeDomainService.ChangeStateType(Id, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识分类-禁用，并且下架知识
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeTypeChangeStateAndOffShelf)]
        [HttpPut("changestateandoffshelf")]
        public async Task TypeChangeStateAndOffShelf(string Id)
        {
            await _knowledgeTypeDomainService.ChangeStateTypeAndOffShelf(Id, HttpContext.RequestAborted);
        }

        /// <summary>
        ///知识分类- 获取层级分类
        /// </summary>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeTypeMange)]
        [HttpGet("treelist")]
        public async Task<List<KnowledgeType>> GetTreeList()
        {
            return await _knowledgeTypeRepository.GetTree(Guid.Empty.ToString());
        }

        /// <summary>
        /// 知识分类- 只获取一级
        /// </summary>
        /// <param name="ParentId">不传默认查询第一级</param>
        /// <returns></returns>
        [HttpGet("typelist")]
        [AllowAnonymous]
        public async Task<List<KnowledgeType>> GetTypeList(string? ParentId)
        {
            if (string.IsNullOrEmpty(ParentId))
                ParentId = Guid.Empty.ToString();
            // return await _knowledgeTypeRepository.QueryAsync(p=>p.ParentId== ParentId);
            return await _knowledgeTypeRepository.Queryable().Where(p => p.ParentId == ParentId).OrderByDescending(p => p.Sort).ToListAsync();
        }

        #endregion

        #region 编写规范
        /// <summary>
        /// 编写规范-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("info-standard/{Id}")]
        public async Task<KnowledgeStandard> GetStandard(string Id)
        {
            if (string.IsNullOrEmpty(Id))
                throw UserFriendlyException.SameMessage("ID不能为空！");

            var sandard = await _knowledgeStandardRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (sandard is null)
                throw UserFriendlyException.SameMessage("编写规范查询失败！");
            return sandard;
        }

        /// <summary>
        /// 编写规范-删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.RemoveKnowledgeStandard)]
        [HttpDelete("remove-standard")]
        public async Task RemoveStandard(string Id)
        {
            if (string.IsNullOrEmpty(Id))
                throw UserFriendlyException.SameMessage("ID不能为空！");
            //删除操作
            await _knowledgeStandardRepository.RemoveAsync(Id, false, HttpContext.RequestAborted);

        }

        /// <summary>
        /// 编写规范-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeStandardAdd)]
        [HttpPost("add-standard")]
        public async Task AddStandard([FromBody] AddKnowledgeStandardDto dto)
        {
            var sandard = _mapper.Map<KnowledgeStandard>(dto);
            await _knowledgeStandardRepository.AddAsync(sandard, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 编写规范-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeStandardUpdate)]
        [HttpPut("update-standard")]
        public async Task UpdateStandard([FromBody] UpdateKnowledgeStandardDto dto)
        {
            var sandard = _mapper.Map<KnowledgeStandard>(dto);
            await _knowledgeStandardRepository.UpdateAsync(sandard, HttpContext.RequestAborted);
        }
        #endregion

    }
}
